@@ -1,4 +1,4 @@
-use super::repo::*;
+use super::{config::*, repo::*};
 use anyhow::*;
 use glob::*;
 use serde::{Deserialize, Serialize};
@@ -56,26 +56,23 @@ impl Database {
         })
     }
 
-    pub fn open_env(
+    pub fn open_env_from_commit(
+        &self,
         path_to_config: &str,
         ignore_queue: bool,
         scope: &str,
-        env_name: &str,
-        propagated_name: Option<&String>,
+        env_config: &EnvironmentConfig,
         commit: CommitHash,
         repo: &Repo,
     ) -> Result<Self> {
         let dir = Self::state_dir_from_config(scope, path_to_config);
-        let env_file = format!("{}/{}.state", dir, env_name);
-        let env_path = Path::new(&env_file);
-        let env_state = repo.get_file_content(commit.clone(), env_path, |bytes| {
-            EnvironmentState::from_reader(bytes)
-        })?;
         let mut state = DbState::default();
-        if let Some(env_state) = env_state {
-            state.environments.insert(env_name.to_string(), env_state);
+        if let Some(env_state) = self.state.environments.get(&env_config.name) {
+            state
+                .environments
+                .insert(env_config.name.to_string(), env_state.clone());
         }
-        if let Some(last_env) = propagated_name {
+        if let Some(last_env) = env_config.propagated_from() {
             let env_file = format!("{}/{}.state", dir, last_env);
             let env_path = Path::new(&env_file);
             if let Some(env_state) = repo.get_file_content(commit, env_path, |bytes| {
@@ -241,7 +238,7 @@ impl DbState {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct EnvironmentState {
     current: DeployState,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -258,7 +255,7 @@ impl EnvironmentState {
     }
 }
 
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct DeployState {
     pub head_commit: CommitHash,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -307,8 +304,8 @@ impl DeployState {
             .files
             .iter()
             .filter_map(|(ident, state)| {
+                removed_files.remove(&ident);
                 if let Some(last_state) = other.files.get(ident) {
-                    removed_files.remove(&ident);
                     if state.file_hash.is_none() && last_state.file_hash.is_none() {
                         None
                     } else if state.dirty
@@ -328,7 +325,6 @@ impl DeployState {
                         None
                     }
                 } else {
-                    removed_files.remove(&ident);
                     Some(FileDiff {
                         ident: ident.clone(),
                         current_state: if state.file_hash.is_some() {
